#Region "Autodesk Copyright"


'      .NET Sample
'
'      Copyright (c) 2008 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
'

#End Region
 
#Region "Namespaces References"

Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Diagnostics
Imports System.Windows.Forms

' AutoCAD MEP Specific
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.GeomeTry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices
Imports Autodesk.Aec.Building.Hvac.DatabaseServices

' create an alias to the AutoCAD Application object
Imports AcadApplication = Autodesk.AutoCAD.ApplicationServices.Application

#End Region
 
<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(AddModifyPart))> 
<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(GetType(AddModifyPart))> 

Public Class AddModifyPart
    Implements IExtensionApplication


#Region "IExtensionApplication"

    Public Sub Initialize() Implements IExtensionApplication.Initialize
        Dim ed As Editor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
        ed.WriteMessage("Loading AddModifyPart..." & vbCrLf)
    End Sub

    Public Sub Terminate() Implements IExtensionApplication.Terminate
    End Sub

#End Region

#Region "Add Part Command"

    <Autodesk.AutoCAD.Runtime.CommandMethod("AddModifyPart", "AddPart", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub AddPart()
        Try
            Dim editor As Editor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
            Dim db As Database = AcadApplication.DocumentManager.MdiActiveDocument.Database
            Dim transMgr As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager

            ' set the prompt options
            Dim prOptions As PromptPointOptions = New PromptPointOptions(vbNewLine + "Specify start point for the duct: ")

            ' issue the prompt
            Dim ptResult As PromptPointResult = editor.GetPoint(prOptions)

            ' nothing to do if the user canceled
            If ptResult.Status = PromptStatus.Cancel Then
                Return
            End If

            ' get the value the user selected
            Dim ptStart As Point3d = ptResult.Value

            ' reset the options for the endpoint
            prOptions.Message = "\nSpecify end point of the duct: "
            prOptions.BasePoint = ptStart
            prOptions.UseDashedLine = True
            prOptions.UseBasePoint = True

            ' issue the prompt
            ptResult = editor.GetPoint(prOptions)

            ' nothing to do if the user cancelled
            If ptResult.Status = PromptStatus.Cancel Then
                Return
            End If

            ' get the value the user selected
            Dim ptEnd As Point3d = ptResult.Value

            ' create a new Duct object
            Dim duct As Duct = New Duct()

            ' make sure to call these before setting your geometry on the part				
            duct.SetToStandard(db)
            duct.SetDatabaseDefaults(db)

            ' the next bit of code shows how to query the catalog for a part
            ' First we create a query object, then add the parameters that we are interested in 
            ' to the query.  Based on the amount of parameters specified, the results of the query 
            ' may contain multiple records, one record, or no records.

            Dim diameter As Double = 12.0

            Dim query As DataQuery = New DataQuery()
            query.Domain = Domain.DuctComponent                                 ' we want a duct
            query.PartGuid = "B7B201F3-FE40-404C-B5BC-0FB413D25FF8"             ' specify a Round duct
            query.AddSizeParameter(Context.ConnectionSizeDiameter, 1, diameter)

            Dim formatter As EnumStringFormatter = New EnumStringFormatter()
            Dim strValue As String = String.Format(formatter, "{0:CatG}", ConnectionType.Slip)
            query.AddSizeParameter(Context.ConnectionPortPhysicalType, 1, strValue)


            ' set the maximum number of records we would like to see
            Dim maxRecords As Integer = 10

            ' get the ExpandedTable from our query
            Dim exTable As DataExpandedTable = PartManager.GetPartTable(query, maxRecords)

            ' get all the records from the table
            Dim recordCol As DataRecordCollection = exTable.DataRecords

            If recordCol.Count > 0 Then
                ' get the first record from the collection
                Dim record As DataRecord = recordCol(0)

                ' this will add the catalog part data to the database object
                PartManager.CreatePartViaRecord(exTable, record, duct)
            Else
                editor.WriteMessage(vbNewLine + "No records found.")
                Return
            End If

            ' set the line segment onto the duct
            '
            Dim lineSeg As LineSegment3d = New LineSegment3d(ptStart, ptEnd)
            duct.SetBaseCurve(lineSeg, Vector3d.ZAxis)

            ' now set the layer for the object
            '
            duct.SetDefaultLayer()

            ' finally we will add the duct to the database ands commit the transaction
            Dim myT As Transaction = transMgr.StartTransaction()

            Dim bt As BlockTable = CType(transMgr.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
            Dim btr As BlockTableRecord = CType(transMgr.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
            btr.AppendEntity(duct)
            transMgr.AddNewlyCreatedDBObject(duct, True)

            ' make sure to commit the transaction
            myT.Commit()

        Catch be As Autodesk.Aec.Building.DatabaseServices.BuildingException
            If be.ErrorCode = ErrorCode.QueryResultNull Then
                MessageBox.Show("The query did not find any parts.")
            End If
            Debug.WriteLine(be.Message)
        Catch ex As System.Exception
            Debug.Write(String.Format(vbNewLine + "Caught the following Exception:{0}", ex.ToString()))
        Finally

        End Try
    End Sub
#End Region

#Region "Modify Part Command"

    ' Simple sample to demonstrate how to modify a database resident part
    '
    <Autodesk.AutoCAD.Runtime.CommandMethod("AddModifyPart", "ModifyPart", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub ModifyPart()
        Dim editor As Editor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
        Dim db As Database = AcadApplication.DocumentManager.MdiActiveDocument.Database
        Dim transMgr As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager

        ' set up the entity prompt options, note that we only want a duct object
        Dim prOptions As PromptEntityOptions = New PromptEntityOptions(vbNewLine + "Select duct to modify: ")
        prOptions.SetRejectMessage(vbNewLine + "Object must be of type Duct.")
        prOptions.AddAllowedClass(GetType(Duct), True)

        Dim trans As Transaction = Nothing

        Try
            ' prompt the user to select a duct
            Dim prResult As PromptEntityResult = editor.GetEntity(prOptions)

            If prResult.Status = PromptStatus.Cancel Then
                Return
            End If

            ' start a new transaction
            trans = transMgr.StartTransaction()

            ' get the ObjectId from the selected Duct 
            Dim oidSelected As Autodesk.AutoCAD.DatabaseServices.ObjectId = prResult.ObjectId

            ' open the Block table and get the BlockTableRecord object from the table
            Dim bt As BlockTable = CType(transMgr.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
            Dim btr As BlockTableRecord = CType(transMgr.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)

            ' open the duct for Write based on the objectId
            Dim duct As Duct = CType(transMgr.GetObject(oidSelected, OpenMode.ForWrite, False), Duct)

            If Not duct Is Nothing Then
                Dim ptTopLeft As Point2d, ptBottomRight As Point2d

                ' get the extent of the profile for the duct
                duct.Profile.Extent(ptTopLeft, ptBottomRight)

                ' get the width of the profile and increase by 2x
                Dim dWidth As Double = (ptBottomRight.X - ptTopLeft.X) * 2

                ' create the query object and fill it in with new parameters
                Dim query As DataQuery = New DataQuery()
                query.Domain = duct.Domain

                ' get the connection type
                Dim contyp As ConnectionType = duct.GetConnectionTypeAtLayoutNode(1)

                ' here we need to get the global catalog string from the enum
                ' we cannot just use ToString for the enum value
                Dim formatter As EnumStringFormatter = New EnumStringFormatter()
                Dim sConType As String = String.Format(formatter, "{0:CatG}", contyp)

                ' now add the new size parameter, connection type, and part guid
                query.AddSizeParameter(Context.ConnectionPortPhysicalType, 1, sConType)
                query.AddSizeParameter(Context.ConnectionSizeDiameter, 1, dWidth)
                query.PartGuid = "B7B201F3-FE40-404C-B5BC-0FB413D25FF8"

                ' set the maximum number of records we would like to see
                Dim maxRecords As Integer = 10

                ' now get the ExpandedTable from our query
                Dim exTable As DataExpandedTable = PartManager.GetPartTable(query, maxRecords)

                ' now get all the records from the table
                Dim recordCol As DataRecordCollection = exTable.DataRecords

                ' now modify the part
                If recordCol.Count > 0 Then
                    Dim record As DataRecord = recordCol(0)
                    PartManager.ModifyPartViaRecord(exTable, record, duct)
                    trans.Commit()
                End If

                duct.Dispose()
            End If
        Catch be As Autodesk.Aec.Building.DatabaseServices.BuildingException
            If be.ErrorCode = ErrorCode.QueryResultNull Then
                MessageBox.Show("The query did not find any parts.")
            End If
            Debug.WriteLine(be.Message)
            trans.Abort()
        Catch e As SystemException
            Debug.WriteLine(e.Message)
            trans.Abort()
        Finally
            trans.Dispose()
        End Try
    End Sub

#End Region

End Class

