#Region "Copyright"
'      .NET Sample
'
'      Copyright (c) 2007 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
#End Region

#Region "Imports"

Imports System
Imports System.Resources
Imports System.Reflection
Imports System.Globalization

Imports Autodesk.Aec.SpaceOffsetRulesManager
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.Aec.Structural.DatabaseServices
Imports Autodesk.Aec.Arch.DatabaseServices

#End Region

<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(Nothing)> 

Namespace AecSpaceOffsetStandardSample

#Region "AecSpaceOffsetStandardSample"

    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    '
    '  Basic Standard
    '  ==============
    '
    '  Description:
    '  ------------
    '
    '  The basic standard provides general rules to offset gross and boundaries
    '  from base boundary which are described below.
    '
    '  Calculation Rules
    '  -----------------
    '
    '  =========================================================================
    '                  | Space          | Bounding | Adjacent | Boundary
    '                  | Classification | Objects  | Space    |
    '  =========================================================================
    '   Net Boundary   | --             | Door     | --       | Panel face of
    '                  |                |          |          | door
    '  -------------------------------------------------------------------------
    '   Net Boundary   | --             | --       | --       | Adjacent face
    '  -------------------------------------------------------------------------
    '   Gross Boundary | --             | --       | None     | Opposite face
    '  -------------------------------------------------------------------------
    '   Gross Boundary | --             | --       | Any      | Centerline
    '  =========================================================================
    '
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

    Public Class AecSpaceOffsetStandardSample
        Inherits AecSpaceOffsetStandard

        '  The constructor sets the properties for this standard
        '
        Public Sub New()

            '  set the properties
            '
            SetName("Sample Standard")
        End Sub

        '  Registers the rules with this standard, so the offset
        '  calculation algorithm can call them for the according boundary type
        '
        Protected Overrides Sub InitRules()
            '  Register the rules with this standard, so the
            '  offset calculation algorithm can call them
            '  for the according boundary type
            NetRules().AppendBoundingOpeningRule(New BoundingOpeningRuleNet())

            '  no usable boundary
            UsableRules().SetSpaceRule(New NullSpace())

            GrossRules().AppendBoundingObjectRule(New BoundingObjectRuleGross())
            GrossRules().AppendBoundingAdjacencyRule(New BoundingAdjacencyRuleGross())
        End Sub

    End Class
#End Region

#Region "BoundingObjectRuleGross"
    Public Class BoundingObjectRuleGross
        Inherits AecBoundingObjectRule

        Public Sub New()

            RegisterType(GetType(kAllTypes))

        End Sub

        Public Overrides Function Apply(ByVal idSpace As ObjectId, ByVal idObject As ObjectId) As Boolean

            Dim result As Boolean = True

            Dim db As Autodesk.AutoCAD.DatabaseServices.Database = idObject.Database

            Using transaction As Autodesk.AutoCAD.DatabaseServices.Transaction = db.TransactionManager.StartTransaction()

                Dim obj As Autodesk.AutoCAD.DatabaseServices.DBObject = transaction.GetObject(idObject, OpenMode.ForRead)
                Try
                    Dim member As Member = CType(obj, Member)
                    Dim memType As MemberType = member.MemberType
                    If memType = MemberType.Column Then
                        result = False
                    End If
                Catch

                End Try

                transaction.Commit()
            End Using

            Return result

        End Function

    End Class
#End Region

#Region "NullSpace"
    Public Class NullSpace
        Inherits AecSpaceRule

        Public Sub New()

            RegisterType(GetType(Space))

        End Sub

        Public Overrides Function Apply(ByVal idSpace As Autodesk.AutoCAD.DatabaseServices.ObjectId) As AecSpaceRuleResult

            Return AecSpaceRuleResult.NoBoundary

        End Function

    End Class
#End Region

#Region "BoundingOpeningRuleNet"
    '  The opening rule for the net offset will
    '  set the boundary to the center of the door panel for all doors
    '
    Public Class BoundingOpeningRuleNet
        Inherits AecBoundingOpeningRule

        Public Sub New()

            RegisterType(GetType(Door))

        End Sub

        Public Overrides Function Apply(ByVal idSpace As ObjectId, ByVal idAdjSpace As ObjectId, ByVal idObject As ObjectId, ByVal idOpening As ObjectId) As AecSpaceOffsetOpeningInfo

            If idAdjSpace.IsNull = False Then
                Dim db As Database = HostApplicationServices.WorkingDatabase
                Using transaction As Transaction = db.TransactionManager.StartTransaction()

                    Dim obj As Autodesk.AutoCAD.DatabaseServices.DBObject = transaction.GetObject(idOpening, OpenMode.ForRead)
                    Try
                        Dim door As Door = CType(obj, Door)
                        If door.Width > 30 Then
                            transaction.Commit()
                            Return New AecSpaceOffsetOpeningInfo(AecSpaceOffsetOpeningType.PanelCenter)
                        End If
                        transaction.Commit()
                    Catch

                    End Try

                End Using
            End If

            Return New AecSpaceOffsetOpeningInfo(AecSpaceOffsetOpeningType.NotApplicable)

        End Function

    End Class
#End Region

#Region "BoundingAdjacencyRuleGross"
    '  The bounding adjacent rule of the gross offset will set all
    '  wall boundaries to the opposite plance if there is no adjacent space.
    '  If there is none, the default setup as set by the bounding plane rule
    '  will be kept
    '
    Public Class BoundingAdjacencyRuleGross
        Inherits AecBoundingAdjacencyRule

        Public Sub New()

            RegisterType(GetType(Space))

        End Sub

        Public Overrides Function Apply(ByVal idSpace As ObjectId, ByVal idObject As ObjectId, ByVal idAdjSpace As ObjectId) As AecSpaceOffsetInfo

            If (idAdjSpace.IsNull) Then
                '  If there is no adjacent space, keep the 'Opposite' offset
                '
                Return New AecSpaceOffsetInfo(AecSpaceOffsetType.Opposite)
            Else

                '  If there is an adjacent space, its center
                '
                Return New AecSpaceOffsetInfo(AecSpaceOffsetType.Center)
            End If

        End Function

    End Class
#End Region

End Namespace
